#include <stdio.h>

#include "options.h"
#include "buffer.h"
#include "dss.h"
#include "signkey.h"
#include "util.h"
#include "libtomcrypt/mpi.h"
#include "libtomcrypt/mycrypt.h"

/* test vectors */
/* from the orig webpage one */
#if 0
const char* DSS_p=
      "d411a4a0e393f6aab0f08b14d18458665b3e4dbdce2544543fe365cf71c86224"
      "12db6e7dd02bbe13d88c58d7263e90236af17ac8a9fe5f249cc81f427fc543f7";

const char* DSS_q=
	"b20db0b101df0c6624fc1392ba55f77d577481e5";

const char* DSS_g=
	"b3085510021f999049a9e7cd3872ce9958186b5007e7adaf25248b58a3dc"
	"4f71781d21f2df89b71747bd54b323bbecc443ec1d3e020dadabbf7822578255c104";

const char* DSS_x=
	"6b2cd935d0192d54e2c942b574c80102c8f8ef67";

const char* DSS_k=
	"79577ddcaafddc038b865b19f8eb1ada8a2838c6";

const char* DSS_kinv =
	"2784e3d672d972a74e22c67f4f4f726ecc751efa";

const char* DSS_y =
	"b32fbec03175791df08c3f861c81df7de7e0cba7f1c4f726"
	"9bb12d6c628784fb742e66ed315754dfe38b5984e94d3725"
	"37f655cb3ea4767c878cbd2d783ee662";
#else
/* from fips186-2-change1.pdf */
const char* DSS_p = 
	"8df2a494492276aa3d25759bb06869cbeac0d83afb8d0cf7cbb8324f0d7882e5"
	"d0762fc5b7210eafc2e9adac32ab7aac49693dfbf83724c2ec0736ee31c80291";

const char* DSS_q =
	"c773218c737ec8ee993b4f2ded30f48edace915f";

const char* DSS_g =
	"626d027839ea0a13413163a55b4cb500299d5522956cefcb3bff10f399ce2c2e"
	"71cb9de5fa24babf58e5b79521925c9cc42e9f6f464b088cc572af53e6d78802";

const char* DSS_x =
	"2070b3223dba372fde1c0ffc7b2e3b498b260614";

const char* DSS_k =
	"358dad571462710f50e254cf1a376b2bdeaadfbf";

const char* DSS_kinv =
	"0d5167298202e49b4116ac104fc3f415ae52f917";

const char* DSS_y =
	"19131871d75b1612a819f29d78d1b0d7346f7aa77bb62a859bfd6c5675da9d21"
	"2d3a36ef1672ef660b8c7c255cc0ec74858fba33f44c06699630a76b030ee333";
#endif


const char* DSS_M = "abc";
const int DSS_M_len = 3;

	  

int main (int argc, char ** argv) {

	dss_key key;
	hash_state hs;
	char hashbuf[20];
	buffer *buf;
	mp_int temp1, temp2, temp3;
	char printbuf[1000];

	key.p = (mp_int*)m_malloc(sizeof(mp_int));
	key.q = (mp_int*)m_malloc(sizeof(mp_int));
	key.g = (mp_int*)m_malloc(sizeof(mp_int));
	key.x = (mp_int*)m_malloc(sizeof(mp_int));
	key.y = (mp_int*)m_malloc(sizeof(mp_int));

	m_mp_init(key.p);
	m_mp_init(key.q);
	m_mp_init(key.g);
	m_mp_init(key.y);
	m_mp_init(key.x);
	m_mp_init(&temp1);
	m_mp_init(&temp2);
	m_mp_init(&temp3);

	sha1_init(&hs);
	sha1_process(&hs, DSS_M, DSS_M_len);
	sha1_done(&hs, hashbuf);

	printf("hashbuf is:\n");
	printhex(hashbuf, 20);
	printf("===\n");

	mp_read_radix(key.p, DSS_p, 16);
	mp_read_radix(key.q, DSS_q, 16);
	mp_read_radix(key.g, DSS_g, 16);
	mp_read_radix(key.x, DSS_x, 16);
	mp_read_radix(key.y, DSS_y, 16);

	/* do stuff for y */
	mp_exptmod(key.g, key.x, key.p, &temp1);
	mp_toradix(&temp1, printbuf, 16);
	printf("calc y is %s\n\n", printbuf);

	mp_toradix(key.y, printbuf, 16);
	printf("real y is %s\n\n", printbuf);

	mp_toradix(key.x, printbuf, 16);
	printf("x is %s\n\n", printbuf);

	mp_toradix(key.p, printbuf, 16);
	printf("p is %s\n\n", printbuf);

	mp_toradix(key.q, printbuf, 16);
	printf("q is %s\n\n", printbuf);

	mp_toradix(key.g, printbuf, 16);
	printf("g is %s\n\n", printbuf);

	mp_sub_d(key.p, 1, &temp1);
	mp_div(&temp1, key.q, &temp2, &temp3);

	mp_toradix(&temp3, printbuf, 16);
	printf("temp3 is %s\n\n", printbuf);

	mp_set(&temp3, 2);
	mp_exptmod(&temp3, &temp2, key.p, &temp1);

	mp_toradix(&temp1, printbuf, 16);
	printf("calc g is %s\n\n", printbuf);


	buf = buf_new(100);
	buf_put_dss_sign(buf, &key, DSS_M, DSS_M_len);
	buf_setpos(buf, 0);
	printhex(buf_getptr(buf, buf->len), buf->len);

	mp_clear(&temp1);
	mp_clear(&temp2);
	mp_clear(&temp3);
	dss_key_free(&key);
}
