#include <stdio.h>
#include <string.h>

#include "options.h"
#include "signkey.h"
#include "util.h"
#include "buffer.h"
#include "random.h"
#include "libtomcrypt/mycrypt.h"

static sign_key * loadhostkeys();

int main(int argc, char ** argv) {

	sign_key * key;
	buffer * buf;
	unsigned char data[20];
	hash_state hs;
	char msghash[SHA1_HASH_SIZE];

	initrandom();
	
	key = loadhostkeys();
	buf = buf_new(1000);

	genrandom(data, sizeof(data));
	
	buf_put_sign(buf, key, MSSH_SIGNKEY_RSA, data, sizeof(data));
	printf("pos is %d\n", buf->pos);

	buf_setpos(buf, 0);
	printf("bufdat start\n");
	printhex(buf->data, buf->len);
	printf("bufdat done\n");

	sha1_init(&hs);
	sha1_process(&hs, data, sizeof(data));
	sha1_done(&hs, msghash);
	printf("intermit hash is \n");
	printhex(msghash, SHA1_HASH_SIZE);

	printf("sizeof data = %d\n", sizeof(data));
	printf("the signature is %s\n", buf_verify(buf, key,
				data, sizeof(data)) ? "good" : "bad");
	buf_free(buf);
	sign_key_free(key);
	return 0;
}

static sign_key * loadhostkeys() {

	sign_key * hostkey;
	buffer *buf;

	TRACE("enter loadhostkeys");
	hostkey = new_sign_key();
	buf = buf_new(2000);
#ifdef MSSH_RSA
	buf_readfile(buf, RSA_PRIV_FILENAME);
	buf_setpos(buf, 0);
	buf_get_priv_key(buf, hostkey);
#endif
#ifdef MSSH_DSS
	buf_setpos(buf, 0);
	buf_readfile(buf, DSS_PRIV_FILENAME);
	buf_setpos(buf, 0);
	buf_get_priv_key(buf, hostkey);
#endif
	buf_burn(buf);
	buf_free(buf);

	TRACE("leave loadhostkeys");
	return hostkey;
}
